from _typeshed import Incomplete

normal: str
comma: str
comma_0: str
currency: str
currency_0: str
percent: str
hyperlink: str
followed_hyperlink: str
title: str
headline_1: str
headline_2: str
headline_3: str
headline_4: str
good: str
bad: str
neutral: str
input: str
output: str
calculation: str
linked_cell: str
check_cell: str
warning: str
note: str
explanatory: str
total: str
accent_1: str
accent_1_20: str
accent_1_40: str
accent_1_60: str
accent_2: str
accent_2_20: str
accent_2_40: str
accent_2_60: str
accent_3: str
accent_3_20: str
accent_3_40: str
accent_3_60: str
accent_4: str
accent_4_20: str
accent_4_40: str
accent_4_60: str
accent_5: str
accent_5_20: str
accent_5_40: str
accent_5_60: str
accent_6: str
accent_6_20: str
accent_6_40: str
accent_6_60: str
pandas_highlight: str
styles: Incomplete



# This file does not exist at runtime. It is a helper file to overload imported functions in openpyxl.xml.functions

from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Iterable, Iterator, Mapping, Sequence
from typing import Any, Protocol, TypeVar, overload
from typing_extensions import TypeAlias
from xml.etree.ElementTree import Element, ElementTree, QName, XMLParser, _FileRead

from openpyxl.chart.axis import ChartLines

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)

# Useful protocols, see import comment in openpyxl/xml/functions.pyi

# Comment from openpyxl.cell.rich_text.py
# Usually an Element() from either lxml or xml.etree (has a 'tag' element)
# lxml.etree._Element
# xml.etree.Element
class _HasTag(Protocol):
    tag: str

class _HasGet(Protocol[_T_co]):
    def get(self, value: str, /) -> _T_co | None: ...

class _HasText(Protocol):
    text: str

class _HasAttrib(Protocol):
    attrib: Iterable[Any]  # AnyOf[dict[str, str], Iterable[tuple[str, str]]]

class _HasTagAndGet(_HasTag, _HasGet[_T_co], Protocol[_T_co]): ...  # noqa: Y046
class _HasTagAndText(_HasTag, _HasText, Protocol): ...  # noqa: Y046
class _HasTagAndTextAndAttrib(_HasTag, _HasText, _HasAttrib, Protocol): ...  # noqa: Y046

class _SupportsFindChartLines(Protocol):
    def find(self, path: str, /) -> ChartLines | None: ...

class _SupportsFindAndIterAndAttribAndText(  # noqa: Y046
    _SupportsFindChartLines, Iterable[Incomplete], _HasAttrib, _HasText, Protocol
): ...
class _SupportsIterAndAttrib(Iterable[Incomplete], _HasAttrib, Protocol): ...  # noqa: Y046
class _SupportsIterAndAttribAndTextAndTag(Iterable[Incomplete], _HasAttrib, _HasText, _HasTag, Protocol): ...  # noqa: Y046
class _SupportsIterAndAttribAndTextAndGet(  # noqa: Y046
    Iterable[Incomplete], _HasAttrib, _HasText, _HasGet[Incomplete], Protocol
): ...

class _ParentElement(Protocol[_T]):
    def makeelement(self, tag: str, attrib: dict[str, str], /) -> _T: ...
    def append(self, element: _T, /) -> object: ...

# from lxml.etree import _Element
_lxml_Element: TypeAlias = Element  # noqa: Y042
# from lxml.etree import _ElementTree
_lxml_ElementTree: TypeAlias = ElementTree  # noqa: Y042
# from lxml.etree import QName
_lxml_QName: TypeAlias = QName  # noqa: Y042

# from xml.etree import fromstring
@overload
def SubElement(parent: _ParentElement[_T], tag: str, attrib: dict[str, str] = ..., **extra: str) -> _T: ...

# from lxml.etree import fromstring
@overload
def SubElement(
    _parent: _lxml_Element,  # This would be preferable as a protocol, but it's a C-Extension
    _tag: str | bytes | _lxml_QName,
    attrib: dict[str, str] | dict[bytes, bytes] | None = ...,
    nsmap: Mapping[str, str] | None = ...,
    **extra: str | bytes,
) -> _lxml_ElementTree: ...

# from xml.etree.ElementTree import fromstring
@overload
def fromstring(text: str | ReadableBuffer, parser: XMLParser | None = None) -> Element: ...

# from lxml.etree import fromstring
# But made partial, removing parser arg
@overload
def fromstring(text: str | bytes, *, base_url: str | bytes = ...) -> _lxml_Element: ...  # type: ignore[overload-overlap]

# from defusedxml.ElementTree import fromstring
@overload
def fromstring(text: str, forbid_dtd: bool = False, forbid_entities: bool = True, forbid_external: bool = True) -> int: ...

# from xml.etree.ElementTree import tostring
# But made partial, removing encoding arg
@overload
def tostring(
    element: Element,
    method: str | None = "xml",
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = ...,
    short_empty_elements: bool = ...,
) -> str: ...

# from lxml.etree import Element
# But made partial, removing encoding arg
@overload
def tostring(
    element_or_tree: _lxml_Element | _lxml_ElementTree,
    method: str = ...,
    xml_declaration: bool = ...,
    pretty_print: bool = ...,
    with_tail: bool = ...,
    standalone: bool = ...,
    doctype: str = ...,
    exclusive: bool = ...,
    with_comments: bool = ...,
    inclusive_ns_prefixes=...,
) -> bytes: ...

# from xml.etree.ElementTree import iterparse
@overload
def iterparse(
    source: _FileRead, events: Sequence[str] | None = None, parser: XMLParser | None = None
) -> Iterator[tuple[str, Any]]: ...

# from defusedxml.ElementTree import iterparse
@overload
def iterparse(
    source: _FileRead,
    events: Sequence[str] | None = None,
    parser: XMLParser | None = None,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> Iterator[tuple[str, Any]]: ...



from _typeshed import ConvertibleToFloat, Incomplete, Unused
from collections.abc import Iterable, Iterator, Sequence as ABCSequence
from typing import ClassVar, Final, Literal
from typing_extensions import TypeAlias

from openpyxl.descriptors import Sequence, Strict
from openpyxl.descriptors.base import Alias, Float, MinMax, NoneSet, Set
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.styles.colors import Color, ColorDescriptor
from openpyxl.xml.functions import Element

from ..xml._functions_overloads import _SupportsIterAndAttribAndTextAndTag

FILL_NONE: Final = "none"
FILL_SOLID: Final = "solid"
FILL_PATTERN_DARKDOWN: Final = "darkDown"
FILL_PATTERN_DARKGRAY: Final = "darkGray"
FILL_PATTERN_DARKGRID: Final = "darkGrid"
FILL_PATTERN_DARKHORIZONTAL: Final = "darkHorizontal"
FILL_PATTERN_DARKTRELLIS: Final = "darkTrellis"
FILL_PATTERN_DARKUP: Final = "darkUp"
FILL_PATTERN_DARKVERTICAL: Final = "darkVertical"
FILL_PATTERN_GRAY0625: Final = "gray0625"
FILL_PATTERN_GRAY125: Final = "gray125"
FILL_PATTERN_LIGHTDOWN: Final = "lightDown"
FILL_PATTERN_LIGHTGRAY: Final = "lightGray"
FILL_PATTERN_LIGHTGRID: Final = "lightGrid"
FILL_PATTERN_LIGHTHORIZONTAL: Final = "lightHorizontal"
FILL_PATTERN_LIGHTTRELLIS: Final = "lightTrellis"
FILL_PATTERN_LIGHTUP: Final = "lightUp"
FILL_PATTERN_LIGHTVERTICAL: Final = "lightVertical"
FILL_PATTERN_MEDIUMGRAY: Final = "mediumGray"

_GradientFillType: TypeAlias = Literal["linear", "path"]
_FillsType: TypeAlias = Literal[
    "solid",
    "darkDown",
    "darkGray",
    "darkGrid",
    "darkHorizontal",
    "darkTrellis",
    "darkUp",
    "darkVertical",
    "gray0625",
    "gray125",
    "lightDown",
    "lightGray",
    "lightGrid",
    "lightHorizontal",
    "lightTrellis",
    "lightUp",
    "lightVertical",
    "mediumGray",
]
fills: Final[tuple[_FillsType, ...]]

class Fill(Serialisable):
    tagname: ClassVar[str]
    @classmethod
    def from_tree(cls, el: Iterable[ABCSequence[_SupportsIterAndAttribAndTextAndTag]]) -> PatternFill | GradientFill | None: ...

class PatternFill(Fill):
    tagname: ClassVar[str]
    __elements__: ClassVar[tuple[str, ...]]
    patternType: NoneSet[_FillsType]
    fill_type: Alias
    fgColor: ColorDescriptor[Literal[False]]
    start_color: Alias
    bgColor: ColorDescriptor[Literal[False]]
    end_color: Alias
    def __init__(
        self,
        patternType: _FillsType | Literal["none"] | None = None,
        fgColor: str | Color = ...,
        bgColor: str | Color = ...,
        fill_type: _FillsType | Literal["none"] | None = None,
        start_color: str | Color | None = None,
        end_color: str | Color | None = None,
    ) -> None: ...
    def to_tree(self, tagname: Unused = None, idx: Unused = None) -> Element: ...  # type: ignore[override]

DEFAULT_EMPTY_FILL: Final[PatternFill]
DEFAULT_GRAY_FILL: Final[PatternFill]

class Stop(Serialisable):
    tagname: ClassVar[str]
    position: MinMax[float, Literal[False]]
    color: Incomplete
    def __init__(self, color, position: ConvertibleToFloat) -> None: ...

class StopList(Sequence):
    expected_type: type[Incomplete]
    def __set__(self, obj: Serialisable | Strict, values) -> None: ...

class GradientFill(Fill):
    tagname: ClassVar[str]
    type: Set[_GradientFillType]
    fill_type: Alias
    degree: Float[Literal[False]]
    left: Float[Literal[False]]
    right: Float[Literal[False]]
    top: Float[Literal[False]]
    bottom: Float[Literal[False]]
    stop: Incomplete
    def __init__(
        self,
        type: _GradientFillType = "linear",
        degree: ConvertibleToFloat = 0,
        left: ConvertibleToFloat = 0,
        right: ConvertibleToFloat = 0,
        top: ConvertibleToFloat = 0,
        bottom: ConvertibleToFloat = 0,
        stop=(),
    ) -> None: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...
    def to_tree(  # type: ignore[override]
        self, tagname: Unused = None, namespace: Unused = None, idx: Unused = None
    ) -> Element: ...



from _typeshed import Incomplete

from .numbers import NUMERIC_TYPES as NUMERIC_TYPES
from .strings import safe_string as safe_string

class DummyCode: ...

string_types: Incomplete

def deprecated(reason): ...



from _typeshed import Incomplete
from re import Pattern
from typing import Final

# Can actually be imported from a mix of xml, lxml, et_xmlfile and defusedxml
# But et_xmlfile is untyped. openpyxl does not directly depend on lxml/lxml-stubs.
# And forcing a dependency on defusedxml felt overkill as it just wraps xml
# So for typing purposes, let's pretend xml is the only dependency.
# Prefer using protocols over these for parameters.
from xml.etree.ElementTree import Element as Element, QName as QName

from ._functions_overloads import (
    SubElement as SubElement,
    _HasTag,
    _HasText,
    fromstring as fromstring,
    iterparse as iterparse,
    tostring as tostring,
)

# from lxml.etree import xmlfile
# from et_xmlfile import xmlfile
xmlfile: Incomplete

NS_REGEX: Final[Pattern[str]]

def localname(node: _HasTag) -> str: ...
def whitespace(node: _HasText) -> None: ...



from _typeshed import ConvertibleToFloat, ConvertibleToInt, Incomplete
from typing import ClassVar, Final, Literal, overload
from typing_extensions import TypeAlias

from openpyxl.descriptors import Strict, Typed
from openpyxl.descriptors.base import Alias, Integer, MinMax, Set, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.nested import EmptyTag, NestedInteger, NestedNoneSet, NestedValue, _NestedNoneSetParam
from openpyxl.descriptors.serialisable import Serialisable

from ..xml._functions_overloads import _HasTagAndGet

_ColorSetType: TypeAlias = Literal[
    "dk1", "lt1", "dk2", "lt2", "accent1", "accent2", "accent3", "accent4", "accent5", "accent6", "hlink", "folHlink"
]
_SystemColorVal: TypeAlias = Literal[
    "scrollBar",
    "background",
    "activeCaption",
    "inactiveCaption",
    "menu",
    "window",
    "windowFrame",
    "menuText",
    "windowText",
    "captionText",
    "activeBorder",
    "inactiveBorder",
    "appWorkspace",
    "highlight",
    "highlightText",
    "btnFace",
    "btnShadow",
    "grayText",
    "btnText",
    "inactiveCaptionText",
    "btnHighlight",
    "3dDkShadow",
    "3dLight",
    "infoText",
    "infoBk",
    "hotLight",
    "gradientActiveCaption",
    "gradientInactiveCaption",
    "menuHighlight",
    "menuBar",
]
_SchemeColors: TypeAlias = Literal[
    "bg1",
    "tx1",
    "bg2",
    "tx2",
    "accent1",
    "accent2",
    "accent3",
    "accent4",
    "accent5",
    "accent6",
    "hlink",
    "folHlink",
    "phClr",
    "dk1",
    "lt1",
    "dk2",
    "lt2",
]
_PresetColors: TypeAlias = Literal[
    "aliceBlue",
    "antiqueWhite",
    "aqua",
    "aquamarine",
    "azure",
    "beige",
    "bisque",
    "black",
    "blanchedAlmond",
    "blue",
    "blueViolet",
    "brown",
    "burlyWood",
    "cadetBlue",
    "chartreuse",
    "chocolate",
    "coral",
    "cornflowerBlue",
    "cornsilk",
    "crimson",
    "cyan",
    "darkBlue",
    "darkCyan",
    "darkGoldenrod",
    "darkGray",
    "darkGrey",
    "darkGreen",
    "darkKhaki",
    "darkMagenta",
    "darkOliveGreen",
    "darkOrange",
    "darkOrchid",
    "darkRed",
    "darkSalmon",
    "darkSeaGreen",
    "darkSlateBlue",
    "darkSlateGray",
    "darkSlateGrey",
    "darkTurquoise",
    "darkViolet",
    "dkBlue",
    "dkCyan",
    "dkGoldenrod",
    "dkGray",
    "dkGrey",
    "dkGreen",
    "dkKhaki",
    "dkMagenta",
    "dkOliveGreen",
    "dkOrange",
    "dkOrchid",
    "dkRed",
    "dkSalmon",
    "dkSeaGreen",
    "dkSlateBlue",
    "dkSlateGray",
    "dkSlateGrey",
    "dkTurquoise",
    "dkViolet",
    "deepPink",
    "deepSkyBlue",
    "dimGray",
    "dimGrey",
    "dodgerBlue",
    "firebrick",
    "floralWhite",
    "forestGreen",
    "fuchsia",
    "gainsboro",
    "ghostWhite",
    "gold",
    "goldenrod",
    "gray",
    "grey",
    "green",
    "greenYellow",
    "honeydew",
    "hotPink",
    "indianRed",
    "indigo",
    "ivory",
    "khaki",
    "lavender",
    "lavenderBlush",
    "lawnGreen",
    "lemonChiffon",
    "lightBlue",
    "lightCoral",
    "lightCyan",
    "lightGoldenrodYellow",
    "lightGray",
    "lightGrey",
    "lightGreen",
    "lightPink",
    "lightSalmon",
    "lightSeaGreen",
    "lightSkyBlue",
    "lightSlateGray",
    "lightSlateGrey",
    "lightSteelBlue",
    "lightYellow",
    "ltBlue",
    "ltCoral",
    "ltCyan",
    "ltGoldenrodYellow",
    "ltGray",
    "ltGrey",
    "ltGreen",
    "ltPink",
    "ltSalmon",
    "ltSeaGreen",
    "ltSkyBlue",
    "ltSlateGray",
    "ltSlateGrey",
    "ltSteelBlue",
    "ltYellow",
    "lime",
    "limeGreen",
    "linen",
    "magenta",
    "maroon",
    "medAquamarine",
    "medBlue",
    "medOrchid",
    "medPurple",
    "medSeaGreen",
    "medSlateBlue",
    "medSpringGreen",
    "medTurquoise",
    "medVioletRed",
    "mediumAquamarine",
    "mediumBlue",
    "mediumOrchid",
    "mediumPurple",
    "mediumSeaGreen",
    "mediumSlateBlue",
    "mediumSpringGreen",
    "mediumTurquoise",
    "mediumVioletRed",
    "midnightBlue",
    "mintCream",
    "mistyRose",
    "moccasin",
    "navajoWhite",
    "navy",
    "oldLace",
    "olive",
    "oliveDrab",
    "orange",
    "orangeRed",
    "orchid",
    "paleGoldenrod",
    "paleGreen",
    "paleTurquoise",
    "paleVioletRed",
    "papayaWhip",
    "peachPuff",
    "peru",
    "pink",
    "plum",
    "powderBlue",
    "purple",
    "red",
    "rosyBrown",
    "royalBlue",
    "saddleBrown",
    "salmon",
    "sandyBrown",
    "seaGreen",
    "seaShell",
    "sienna",
    "silver",
    "skyBlue",
    "slateBlue",
    "slateGray",
    "slateGrey",
    "snow",
    "springGreen",
    "steelBlue",
    "tan",
    "teal",
    "thistle",
    "tomato",
    "turquoise",
    "violet",
    "wheat",
    "white",
    "whiteSmoke",
    "yellow",
    "yellowGreen",
]

PRESET_COLORS: Final[list[_PresetColors]]
SCHEME_COLORS: Final[list[_SchemeColors]]

class Transform(Serialisable): ...

class SystemColor(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    tint: NestedInteger[Literal[True]]
    shade: NestedInteger[Literal[True]]
    comp: Typed[Transform, Literal[True]]
    inv: Typed[Transform, Literal[True]]
    gray: Typed[Transform, Literal[True]]
    alpha: NestedInteger[Literal[True]]
    alphaOff: NestedInteger[Literal[True]]
    alphaMod: NestedInteger[Literal[True]]
    hue: NestedInteger[Literal[True]]
    hueOff: NestedInteger[Literal[True]]
    hueMod: NestedInteger[Literal[True]]
    sat: NestedInteger[Literal[True]]
    satOff: NestedInteger[Literal[True]]
    satMod: NestedInteger[Literal[True]]
    lum: NestedInteger[Literal[True]]
    lumOff: NestedInteger[Literal[True]]
    lumMod: NestedInteger[Literal[True]]
    red: NestedInteger[Literal[True]]
    redOff: NestedInteger[Literal[True]]
    redMod: NestedInteger[Literal[True]]
    green: NestedInteger[Literal[True]]
    greenOff: NestedInteger[Literal[True]]
    greenMod: NestedInteger[Literal[True]]
    blue: NestedInteger[Literal[True]]
    blueOff: NestedInteger[Literal[True]]
    blueMod: NestedInteger[Literal[True]]
    gamma: Typed[Transform, Literal[True]]
    invGamma: Typed[Transform, Literal[True]]
    val: Set[_SystemColorVal]
    lastClr: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        val: _SystemColorVal = "windowText",
        lastClr: Incomplete | None = None,
        tint: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        shade: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        comp: Transform | None = None,
        inv: Transform | None = None,
        gray: Transform | None = None,
        alpha: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        alphaOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        alphaMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        hue: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        hueOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        hueMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        sat: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        satOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        satMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        lum: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        lumOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        lumMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        red: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        redOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        redMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        green: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        greenOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        greenMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        blue: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        blueOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        blueMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        gamma: Transform | None = None,
        invGamma: Transform | None = None,
    ) -> None: ...

class HSLColor(Serialisable):
    tagname: ClassVar[str]
    hue: Integer[Literal[False]]
    sat: MinMax[float, Literal[False]]
    lum: MinMax[float, Literal[False]]
    def __init__(self, hue: ConvertibleToInt, sat: ConvertibleToFloat, lum: ConvertibleToFloat) -> None: ...

class RGBPercent(Serialisable):
    tagname: ClassVar[str]
    r: MinMax[float, Literal[False]]
    g: MinMax[float, Literal[False]]
    b: MinMax[float, Literal[False]]
    def __init__(self, r: ConvertibleToFloat, g: ConvertibleToFloat, b: ConvertibleToFloat) -> None: ...

_RGBPercent: TypeAlias = RGBPercent

class SchemeColor(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    tint: NestedInteger[Literal[True]]
    shade: NestedInteger[Literal[True]]
    comp: EmptyTag[Literal[True]]
    inv: NestedInteger[Literal[True]]
    gray: NestedInteger[Literal[True]]
    alpha: NestedInteger[Literal[True]]
    alphaOff: NestedInteger[Literal[True]]
    alphaMod: NestedInteger[Literal[True]]
    hue: NestedInteger[Literal[True]]
    hueOff: NestedInteger[Literal[True]]
    hueMod: NestedInteger[Literal[True]]
    sat: NestedInteger[Literal[True]]
    satOff: NestedInteger[Literal[True]]
    satMod: NestedInteger[Literal[True]]
    lum: NestedInteger[Literal[True]]
    lumOff: NestedInteger[Literal[True]]
    lumMod: NestedInteger[Literal[True]]
    red: NestedInteger[Literal[True]]
    redOff: NestedInteger[Literal[True]]
    redMod: NestedInteger[Literal[True]]
    green: NestedInteger[Literal[True]]
    greenOff: NestedInteger[Literal[True]]
    greenMod: NestedInteger[Literal[True]]
    blue: NestedInteger[Literal[True]]
    blueOff: NestedInteger[Literal[True]]
    blueMod: NestedInteger[Literal[True]]
    gamma: EmptyTag[Literal[True]]
    invGamma: EmptyTag[Literal[True]]
    val: Set[_SchemeColors]
    __elements__: ClassVar[tuple[str, ...]]
    @overload
    def __init__(
        self,
        tint: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        shade: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        comp: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        inv: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        gray: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        alpha: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        alphaOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        alphaMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        hue: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        hueOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        hueMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        sat: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        satOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        satMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        lum: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        lumOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        lumMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        red: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        redOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        redMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        green: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        greenOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        greenMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        blue: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        blueOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        blueMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        gamma: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        invGamma: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        *,
        val: _SchemeColors,
    ) -> None: ...
    @overload
    def __init__(
        self,
        tint: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        shade: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        comp: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None,
        inv: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        gray: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        alpha: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        alphaOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        alphaMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        hue: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        hueOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        hueMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        sat: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        satOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        satMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        lum: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        lumOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        lumMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        red: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        redOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        redMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        green: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        greenOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        greenMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        blue: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        blueOff: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        blueMod: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None,
        gamma: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None,
        invGamma: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None,
        val: _SchemeColors,
    ) -> None: ...

class ColorChoice(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    scrgbClr: Typed[_RGBPercent, Literal[True]]
    RGBPercent: Alias
    srgbClr: NestedValue[_RGBPercent, Literal[True]]
    RGB: Alias
    hslClr: Typed[HSLColor, Literal[True]]
    sysClr: Typed[SystemColor, Literal[True]]
    schemeClr: Typed[SystemColor, Literal[True]]
    prstClr: NestedNoneSet[_PresetColors]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        scrgbClr: _RGBPercent | None = None,
        srgbClr: _HasTagAndGet[_RGBPercent | None] | _RGBPercent | None = None,
        hslClr: HSLColor | None = None,
        sysClr: SystemColor | None = None,
        schemeClr: SystemColor | None = None,
        prstClr: _NestedNoneSetParam[_PresetColors] = None,
    ) -> None: ...

_COLOR_SET: Final[tuple[_ColorSetType, ...]]

class ColorMapping(Serialisable):
    tagname: ClassVar[str]
    bg1: Set[_ColorSetType]
    tx1: Set[_ColorSetType]
    bg2: Set[_ColorSetType]
    tx2: Set[_ColorSetType]
    accent1: Set[_ColorSetType]
    accent2: Set[_ColorSetType]
    accent3: Set[_ColorSetType]
    accent4: Set[_ColorSetType]
    accent5: Set[_ColorSetType]
    accent6: Set[_ColorSetType]
    hlink: Set[_ColorSetType]
    folHlink: Set[_ColorSetType]
    extLst: Typed[ExtensionList, Literal[True]]
    def __init__(
        self,
        bg1: str = "lt1",
        tx1: str = "dk1",
        bg2: str = "lt2",
        tx2: str = "dk2",
        accent1: str = "accent1",
        accent2: str = "accent2",
        accent3: str = "accent3",
        accent4: str = "accent4",
        accent5: str = "accent5",
        accent6: str = "accent6",
        hlink: str = "hlink",
        folHlink: str = "folHlink",
        extLst: ExtensionList | None = None,
    ) -> None: ...

class ColorChoiceDescriptor(Typed[ColorChoice, Literal[True]]):
    expected_type: type[ColorChoice]
    allow_none: Literal[True]
    def __init__(self, name: str | None = None) -> None: ...
    def __set__(self, instance: Serialisable | Strict, value: str | ColorChoice | None) -> None: ...



from _typeshed import Incomplete, Unused

def etree_write_cell(xf, worksheet: Unused, cell, styled: Incomplete | None = None) -> None: ...
def lxml_write_cell(xf, worksheet: Unused, cell, styled: bool = False) -> None: ...

write_cell = lxml_write_cell
write_cell = etree_write_cell



from _typeshed import ConvertibleToFloat, Incomplete, Unused
from typing import ClassVar, Literal, overload
from typing_extensions import TypeAlias

from openpyxl.chart._3d import Surface, View3D
from openpyxl.chart.legend import Legend
from openpyxl.chart.pivot import PivotSource
from openpyxl.chart.plotarea import PlotArea
from openpyxl.chart.print_settings import PrintSettings
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText
from openpyxl.chart.title import Title
from openpyxl.descriptors.base import Alias, String, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.nested import NestedBool, NestedMinMax, NestedNoneSet, NestedString, _NestedNoneSetParam
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.drawing.colors import ColorMapping
from openpyxl.xml.functions import Element

from ..xml._functions_overloads import _HasTagAndGet

_ChartContainerDispBlanksAs: TypeAlias = Literal["span", "gap", "zero"]

class ChartContainer(Serialisable):
    tagname: ClassVar[str]
    title: Typed[Title, Literal[True]]
    autoTitleDeleted: NestedBool[Literal[True]]
    pivotFmts: Incomplete

    # Same as _3DBase
    # https://github.com/python/mypy/issues/6700
    view3D: Typed[View3D, Literal[True]]
    floor: Typed[Surface, Literal[True]]
    sideWall: Typed[Surface, Literal[True]]
    backWall: Typed[Surface, Literal[True]]

    plotArea: Typed[PlotArea, Literal[False]]
    legend: Typed[Legend, Literal[True]]
    plotVisOnly: NestedBool[Literal[False]]
    dispBlanksAs: NestedNoneSet[_ChartContainerDispBlanksAs]
    showDLblsOverMax: NestedBool[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        title: Title | None = None,
        autoTitleDeleted: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        pivotFmts=(),
        view3D: View3D | None = None,
        floor: Surface | None = None,
        sideWall: Surface | None = None,
        backWall: Surface | None = None,
        plotArea: PlotArea | None = None,
        legend: Legend | None = None,
        plotVisOnly: _HasTagAndGet[_ConvertibleToBool] | _ConvertibleToBool = True,
        dispBlanksAs: _NestedNoneSetParam[_ChartContainerDispBlanksAs] = "gap",
        showDLblsOverMax: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        extLst: Unused = None,
    ) -> None: ...

class Protection(Serialisable):
    tagname: ClassVar[str]
    chartObject: NestedBool[Literal[True]]
    data: NestedBool[Literal[True]]
    formatting: NestedBool[Literal[True]]
    selection: NestedBool[Literal[True]]
    userInterface: NestedBool[Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        chartObject: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        data: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        formatting: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        selection: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        userInterface: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
    ) -> None: ...

class ExternalData(Serialisable):
    tagname: ClassVar[str]
    autoUpdate: NestedBool[Literal[True]]
    id: String[Literal[False]]
    @overload
    def __init__(
        self, autoUpdate: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None, *, id: str
    ) -> None: ...
    @overload
    def __init__(self, autoUpdate: Incomplete | None, id: str) -> None: ...

class ChartSpace(Serialisable):
    tagname: ClassVar[str]
    date1904: NestedBool[Literal[True]]
    lang: NestedString[Literal[True]]
    roundedCorners: NestedBool[Literal[True]]
    style: NestedMinMax[float, Literal[True]]
    clrMapOvr: Typed[ColorMapping, Literal[True]]
    pivotSource: Typed[PivotSource, Literal[True]]
    protection: Typed[Protection, Literal[True]]
    chart: Typed[ChartContainer, Literal[False]]
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphical_properties: Alias
    txPr: Typed[RichText, Literal[True]]
    textProperties: Alias
    externalData: Typed[ExternalData, Literal[True]]
    printSettings: Typed[PrintSettings, Literal[True]]
    userShapes: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    @overload
    def __init__(
        self,
        date1904: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        lang: object = None,
        roundedCorners: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        style: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None = None,
        clrMapOvr: ColorMapping | None = None,
        pivotSource: PivotSource | None = None,
        protection: Protection | None = None,
        *,
        chart: ChartContainer,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        externalData: ExternalData | None = None,
        printSettings: PrintSettings | None = None,
        userShapes: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        date1904: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None,
        lang: object,
        roundedCorners: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None,
        style: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None,
        clrMapOvr: ColorMapping | None,
        pivotSource: PivotSource | None,
        protection: Protection | None,
        chart: ChartContainer,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        externalData: ExternalData | None = None,
        printSettings: PrintSettings | None = None,
        userShapes: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...
    def to_tree(self, tagname: Unused = None, idx: Unused = None, namespace: Unused = None) -> Element: ...



from _typeshed import ConvertibleToFloat, ConvertibleToInt, Unused
from typing import ClassVar, Literal
from typing_extensions import TypeAlias

from openpyxl.chart.data_source import NumFmt
from openpyxl.chart.layout import Layout
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText, Text
from openpyxl.descriptors.base import Alias, String, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.nested import NestedBool, NestedFloat, NestedInteger, NestedSet
from openpyxl.descriptors.serialisable import Serialisable

from ..xml._functions_overloads import _HasTagAndGet

_TrendlineTrendlineType: TypeAlias = Literal["exp", "linear", "log", "movingAvg", "poly", "power"]

class TrendlineLabel(Serialisable):
    tagname: ClassVar[str]
    layout: Typed[Layout, Literal[True]]
    tx: Typed[Text, Literal[True]]
    numFmt: Typed[NumFmt, Literal[True]]
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    txPr: Typed[RichText, Literal[True]]
    textProperties: Alias
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        layout: Layout | None = None,
        tx: Text | None = None,
        numFmt: NumFmt | None = None,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        extLst: Unused = None,
    ) -> None: ...

class Trendline(Serialisable):
    tagname: ClassVar[str]
    name: String[Literal[True]]
    spPr: Typed[ExtensionList, Literal[True]]
    graphicalProperties: Alias
    trendlineType: NestedSet[_TrendlineTrendlineType]
    order: NestedInteger[Literal[True]]
    period: NestedInteger[Literal[True]]
    forward: NestedFloat[Literal[True]]
    backward: NestedFloat[Literal[True]]
    intercept: NestedFloat[Literal[True]]
    dispRSqr: NestedBool[Literal[True]]
    dispEq: NestedBool[Literal[True]]
    trendlineLbl: Typed[ExtensionList, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        name: str | None = None,
        spPr: ExtensionList | None = None,
        trendlineType: _HasTagAndGet[_TrendlineTrendlineType] | _TrendlineTrendlineType = "linear",
        order: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        period: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        forward: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None = None,
        backward: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None = None,
        intercept: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None = None,
        dispRSqr: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        dispEq: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        trendlineLbl: ExtensionList | None = None,
        extLst: Unused = None,
    ) -> None: ...



from collections.abc import Iterable

from openpyxl import _Decodable
from openpyxl.cell.cell import Cell
from openpyxl.workbook.child import _WorkbookChild
from openpyxl.workbook.workbook import Workbook
from openpyxl.worksheet.table import TableList
from openpyxl.worksheet.views import SheetView
from openpyxl.worksheet.worksheet import Worksheet

class WriteOnlyWorksheet(_WorkbookChild):
    mime_type = Worksheet.mime_type
    add_chart = Worksheet.add_chart
    add_image = Worksheet.add_image
    add_table = Worksheet.add_table

    # Same properties as Worksheet
    # https://github.com/python/mypy/issues/6700
    @property
    def tables(self) -> TableList: ...
    @property
    def print_titles(self) -> str: ...
    @property
    def print_title_cols(self) -> str | None: ...
    @print_title_cols.setter
    def print_title_cols(self, cols: str | None) -> None: ...
    @property
    def print_title_rows(self) -> str | None: ...
    @print_title_rows.setter
    def print_title_rows(self, rows: str | None) -> None: ...
    @property
    def freeze_panes(self) -> str | None: ...
    @freeze_panes.setter
    def freeze_panes(self, topLeftCell: str | Cell | None = ...) -> None: ...
    @property
    def print_area(self) -> str: ...
    @print_area.setter
    def print_area(self, value: str | Iterable[str] | None) -> None: ...
    @property
    def sheet_view(self) -> SheetView: ...
    def __init__(self, parent: Workbook | None, title: str | _Decodable | None) -> None: ...
    @property
    def closed(self) -> bool: ...
    def close(self) -> None: ...
    def append(self, row) -> None: ...



from _typeshed import ConvertibleToFloat, ConvertibleToInt
from typing import ClassVar, Final, Literal
from typing_extensions import Self, TypeAlias

from openpyxl.descriptors.base import Alias, _ConvertibleToBool
from openpyxl.descriptors.nested import (
    NestedBool,
    NestedFloat,
    NestedInteger,
    NestedMinMax,
    NestedNoneSet,
    NestedString,
    _NestedNoneSetParam,
)
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.styles.colors import Color, ColorDescriptor

from ..xml._functions_overloads import _HasTagAndGet, _SupportsFindAndIterAndAttribAndText

_FontU: TypeAlias = Literal["single", "double", "singleAccounting", "doubleAccounting"]
_FontVertAlign: TypeAlias = Literal["superscript", "subscript", "baseline"]
_FontScheme: TypeAlias = Literal["major", "minor"]

class Font(Serialisable):
    UNDERLINE_DOUBLE: Final = "double"
    UNDERLINE_DOUBLE_ACCOUNTING: Final = "doubleAccounting"
    UNDERLINE_SINGLE: Final = "single"
    UNDERLINE_SINGLE_ACCOUNTING: Final = "singleAccounting"
    name: NestedString[Literal[True]]
    charset: NestedInteger[Literal[True]]
    family: NestedMinMax[float, Literal[True]]
    sz: NestedFloat[Literal[True]]
    size: Alias
    b: NestedBool[Literal[False]]
    bold: Alias
    i: NestedBool[Literal[False]]
    italic: Alias
    strike: NestedBool[Literal[True]]
    strikethrough: Alias
    outline: NestedBool[Literal[True]]
    shadow: NestedBool[Literal[True]]
    condense: NestedBool[Literal[True]]
    extend: NestedBool[Literal[True]]
    u: NestedNoneSet[_FontU]
    underline: Alias
    vertAlign: NestedNoneSet[_FontVertAlign]
    color: ColorDescriptor[Literal[True]]
    scheme: NestedNoneSet[_FontScheme]
    tagname: ClassVar[str]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        name: object = None,
        sz: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None = None,
        b: _HasTagAndGet[_ConvertibleToBool] | _ConvertibleToBool = None,
        i: _HasTagAndGet[_ConvertibleToBool] | _ConvertibleToBool = None,
        charset: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        u: _NestedNoneSetParam[_FontU] = None,
        strike: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        color: str | Color | None = None,
        scheme: _NestedNoneSetParam[_FontScheme] = None,
        family: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None = None,
        size: _HasTagAndGet[ConvertibleToFloat] | ConvertibleToFloat | None = None,
        bold: _HasTagAndGet[_ConvertibleToBool] | _ConvertibleToBool | None = None,
        italic: _HasTagAndGet[_ConvertibleToBool] | _ConvertibleToBool | None = None,
        strikethrough: _HasTagAndGet[_ConvertibleToBool] | _ConvertibleToBool | None = None,
        underline: _NestedNoneSetParam[_FontU] = None,
        vertAlign: _NestedNoneSetParam[_FontVertAlign] = None,
        outline: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        shadow: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        condense: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        extend: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
    ) -> None: ...
    @classmethod
    def from_tree(cls, node: _SupportsFindAndIterAndAttribAndText) -> Self: ...

DEFAULT_FONT: Final[Font]



from collections.abc import Iterable
from typing import TypeVar

_T = TypeVar("_T")

class IndexedList(list[_T]):
    clean: bool
    def __init__(self, iterable: Iterable[_T] | None = None) -> None: ...
    def __contains__(self, value: object) -> bool: ...
    def index(self, value: _T): ...  # type: ignore[override]
    def append(self, value: _T) -> None: ...
    def add(self, value: _T): ...



from _typeshed import ConvertibleToInt, Incomplete, Unused
from typing import ClassVar, Literal
from typing_extensions import TypeAlias

from openpyxl.descriptors.base import Bool, Integer, Set, String, _ConvertibleToBool
from openpyxl.descriptors.serialisable import Serialisable

_TableStyleElementType: TypeAlias = Literal[
    "wholeTable",
    "headerRow",
    "totalRow",
    "firstColumn",
    "lastColumn",
    "firstRowStripe",
    "secondRowStripe",
    "firstColumnStripe",
    "secondColumnStripe",
    "firstHeaderCell",
    "lastHeaderCell",
    "firstTotalCell",
    "lastTotalCell",
    "firstSubtotalColumn",
    "secondSubtotalColumn",
    "thirdSubtotalColumn",
    "firstSubtotalRow",
    "secondSubtotalRow",
    "thirdSubtotalRow",
    "blankRow",
    "firstColumnSubheading",
    "secondColumnSubheading",
    "thirdColumnSubheading",
    "firstRowSubheading",
    "secondRowSubheading",
    "thirdRowSubheading",
    "pageFieldLabels",
    "pageFieldValues",
]

class TableStyleElement(Serialisable):
    tagname: ClassVar[str]
    type: Set[_TableStyleElementType]
    size: Integer[Literal[True]]
    dxfId: Integer[Literal[True]]
    def __init__(
        self, type: _TableStyleElementType, size: ConvertibleToInt | None = None, dxfId: ConvertibleToInt | None = None
    ) -> None: ...

class TableStyle(Serialisable):
    tagname: ClassVar[str]
    name: String[Literal[False]]
    pivot: Bool[Literal[True]]
    table: Bool[Literal[True]]
    count: Integer[Literal[True]]
    tableStyleElement: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        name: str,
        pivot: _ConvertibleToBool | None = None,
        table: _ConvertibleToBool | None = None,
        count: ConvertibleToInt | None = None,
        tableStyleElement=(),
    ) -> None: ...

class TableStyleList(Serialisable):
    tagname: ClassVar[str]
    defaultTableStyle: String[Literal[True]]
    defaultPivotStyle: String[Literal[True]]
    tableStyle: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        count: Unused = None,
        defaultTableStyle: str | None = "TableStyleMedium9",
        defaultPivotStyle: str | None = "PivotStyleLight16",
        tableStyle=(),
    ) -> None: ...
    @property
    def count(self) -> int: ...



from xml.etree.ElementTree import _FileRead

from openpyxl.cell.rich_text import CellRichText

def read_string_table(xml_source: _FileRead) -> list[str]: ...
def read_rich_text(xml_source: _FileRead) -> list[CellRichText | str]: ...



from collections.abc import Iterable
from re import Pattern
from typing import Final

from openpyxl import _Decodable
from openpyxl.workbook.workbook import Workbook
from openpyxl.worksheet.header_footer import HeaderFooter, HeaderFooterItem

INVALID_TITLE_REGEX: Final[Pattern[str]]

def avoid_duplicate_name(names: Iterable[str], value: str) -> str: ...

class _WorkbookChild:
    HeaderFooter: HeaderFooter
    def __init__(self, parent: Workbook | None = None, title: str | _Decodable | None = None) -> None: ...
    @property
    def parent(self) -> Workbook | None: ...
    @property
    def encoding(self) -> str: ...  # Will error without a parent.
    @property
    def title(self) -> str: ...
    @title.setter
    def title(self, value: str | _Decodable) -> None: ...
    @property
    def oddHeader(self) -> HeaderFooterItem | None: ...
    @oddHeader.setter
    def oddHeader(self, value: HeaderFooterItem | None) -> None: ...
    @property
    def oddFooter(self) -> HeaderFooterItem | None: ...
    @oddFooter.setter
    def oddFooter(self, value: HeaderFooterItem | None) -> None: ...
    @property
    def evenHeader(self) -> HeaderFooterItem | None: ...
    @evenHeader.setter
    def evenHeader(self, value: HeaderFooterItem | None) -> None: ...
    @property
    def evenFooter(self) -> HeaderFooterItem | None: ...
    @evenFooter.setter
    def evenFooter(self, value: HeaderFooterItem | None) -> None: ...
    @property
    def firstHeader(self) -> HeaderFooterItem | None: ...
    @firstHeader.setter
    def firstHeader(self, value: HeaderFooterItem | None) -> None: ...
    @property
    def firstFooter(self) -> HeaderFooterItem | None: ...
    @firstFooter.setter
    def firstFooter(self, value: HeaderFooterItem | None) -> None: ...
    @property
    def path(self) -> str: ...



from _typeshed import ConvertibleToInt, Incomplete, Unused
from array import array
from collections.abc import Iterable, MutableSequence
from typing import ClassVar, Generic, Literal, TypeVar
from typing_extensions import Self

from openpyxl.descriptors.base import Bool, Integer, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.styles.alignment import Alignment
from openpyxl.styles.protection import Protection

_T = TypeVar("_T")

class ArrayDescriptor(Generic[_T]):
    key: int
    def __init__(self, key: int) -> None: ...
    def __get__(self, instance: MutableSequence[_T], cls: Unused) -> _T: ...
    def __set__(self, instance: MutableSequence[_T], value: _T) -> None: ...

class StyleArray(array[int]):
    tagname: ClassVar[str]
    fontId: ArrayDescriptor[int]
    fillId: ArrayDescriptor[int]
    borderId: ArrayDescriptor[int]
    numFmtId: ArrayDescriptor[int]
    protectionId: ArrayDescriptor[int]
    alignmentId: ArrayDescriptor[int]
    pivotButton: ArrayDescriptor[int]
    quotePrefix: ArrayDescriptor[int]
    xfId: ArrayDescriptor[int]
    def __new__(cls, args: bytes | bytearray | Iterable[int] = [0, 0, 0, 0, 0, 0, 0, 0, 0]) -> Self: ...
    def __init__(self, args: bytes | bytearray | Iterable[int] = [0, 0, 0, 0, 0, 0, 0, 0, 0]) -> None: ...
    def __hash__(self) -> int: ...
    def __copy__(self) -> StyleArray: ...
    def __deepcopy__(self, memo: Unused) -> StyleArray: ...

class CellStyle(Serialisable):
    tagname: ClassVar[str]
    numFmtId: Integer[Literal[False]]
    fontId: Integer[Literal[False]]
    fillId: Integer[Literal[False]]
    borderId: Integer[Literal[False]]
    xfId: Integer[Literal[True]]
    quotePrefix: Bool[Literal[True]]
    pivotButton: Bool[Literal[True]]
    applyNumberFormat: Bool[Literal[True]]
    applyFont: Bool[Literal[True]]
    applyFill: Bool[Literal[True]]
    applyBorder: Bool[Literal[True]]
    # Overwritten by properties below
    # applyAlignment: Bool[Literal[True]]
    # applyProtection: Bool[Literal[True]]
    alignment: Typed[Alignment, Literal[True]]
    protection: Typed[Protection, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        numFmtId: ConvertibleToInt = 0,
        fontId: ConvertibleToInt = 0,
        fillId: ConvertibleToInt = 0,
        borderId: ConvertibleToInt = 0,
        xfId: ConvertibleToInt | None = None,
        quotePrefix: _ConvertibleToBool | None = None,
        pivotButton: _ConvertibleToBool | None = None,
        applyNumberFormat: _ConvertibleToBool | None = None,
        applyFont: _ConvertibleToBool | None = None,
        applyFill: _ConvertibleToBool | None = None,
        applyBorder: _ConvertibleToBool | None = None,
        applyAlignment: Unused = None,
        applyProtection: Unused = None,
        alignment: Alignment | None = None,
        protection: Protection | None = None,
        extLst: Unused = None,
    ) -> None: ...
    def to_array(self): ...
    @classmethod
    def from_array(cls, style): ...
    @property
    def applyProtection(self) -> Literal[True] | None: ...
    @property
    def applyAlignment(self) -> Literal[True] | None: ...

class CellStyleList(Serialisable):
    tagname: ClassVar[str]
    __attrs__: ClassVar[tuple[str, ...]]
    # Overwritten by property below
    # count: Integer
    xf: Incomplete
    alignment: Incomplete
    protection: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, count: Unused = None, xf=()) -> None: ...
    @property
    def count(self) -> int: ...
    def __getitem__(self, idx): ...



from _typeshed import Incomplete
from typing import ClassVar, Literal

from . import Integer, MatchPattern, MinMax, Strict, String
from .base import _M, _N
from .serialisable import Serialisable

class HexBinary(MatchPattern[str, Incomplete]):
    pattern: str

class UniversalMeasure(MatchPattern[str, Incomplete]):
    pattern: str

class TextPoint(MinMax[_M, _N]):
    expected_type: type[_M]
    min: float
    max: float

Coordinate = Integer

class Percentage(MinMax[float, Incomplete]):
    pattern: str
    min: float
    max: float
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...

class Extension(Serialisable):
    uri: String[Literal[False]]
    def __init__(self, uri: str) -> None: ...

class ExtensionList(Serialisable):
    ext: Incomplete
    def __init__(self, ext=()) -> None: ...

class Relation(String[Incomplete]):
    namespace: ClassVar[str]
    allow_none: bool

class Base64Binary(MatchPattern[str, Incomplete]):
    pattern: str

class Guid(MatchPattern[str, Incomplete]):
    pattern: str

class CellRange(MatchPattern[str, Incomplete]):
    pattern: str
    allow_none: bool
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...



from setuptools import setup

name = "types-openpyxl"
description = "Typing stubs for openpyxl"
long_description = '''
## Typing stubs for openpyxl

This is a [PEP 561](https://peps.python.org/pep-0561/)
type stub package for the [`openpyxl`](https://foss.heptapod.net/openpyxl/openpyxl) package.
It can be used by type-checking tools like
[mypy](https://github.com/python/mypy/),
[pyright](https://github.com/microsoft/pyright),
[pytype](https://github.com/google/pytype/),
PyCharm, etc. to check code that uses
`openpyxl`.

This version of `types-openpyxl` aims to provide accurate annotations
for `openpyxl==3.1.*`.
The source for this package can be found at
https://github.com/python/typeshed/tree/main/stubs/openpyxl. All fixes for
types and metadata should be contributed there.

See https://github.com/python/typeshed/blob/main/README.md for more details.
This package was generated from typeshed commit `273e9ea6ddc2d5cad2db7dd25a0383a61be15675` and was tested
with mypy 1.10.0, pyright 1.1.360, and
pytype 2024.4.11.
'''.lstrip()

setup(name=name,
      version="3.1.0.20240425",
      description=description,
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/python/typeshed",
      project_urls={
          "GitHub": "https://github.com/python/typeshed",
          "Changes": "https://github.com/typeshed-internal/stub_uploader/blob/main/data/changelogs/openpyxl.md",
          "Issue tracker": "https://github.com/python/typeshed/issues",
          "Chat": "https://gitter.im/python/typing",
      },
      install_requires=[],
      packages=['openpyxl-stubs'],
      package_data={'openpyxl-stubs': ['__init__.pyi', '_constants.pyi', 'cell/__init__.pyi', 'cell/_writer.pyi', 'cell/cell.pyi', 'cell/read_only.pyi', 'cell/rich_text.pyi', 'cell/text.pyi', 'chart/_3d.pyi', 'chart/__init__.pyi', 'chart/_chart.pyi', 'chart/area_chart.pyi', 'chart/axis.pyi', 'chart/bar_chart.pyi', 'chart/bubble_chart.pyi', 'chart/chartspace.pyi', 'chart/data_source.pyi', 'chart/descriptors.pyi', 'chart/error_bar.pyi', 'chart/label.pyi', 'chart/layout.pyi', 'chart/legend.pyi', 'chart/line_chart.pyi', 'chart/marker.pyi', 'chart/picture.pyi', 'chart/pie_chart.pyi', 'chart/pivot.pyi', 'chart/plotarea.pyi', 'chart/print_settings.pyi', 'chart/radar_chart.pyi', 'chart/reader.pyi', 'chart/reference.pyi', 'chart/scatter_chart.pyi', 'chart/series.pyi', 'chart/series_factory.pyi', 'chart/shapes.pyi', 'chart/stock_chart.pyi', 'chart/surface_chart.pyi', 'chart/text.pyi', 'chart/title.pyi', 'chart/trendline.pyi', 'chart/updown_bars.pyi', 'chartsheet/__init__.pyi', 'chartsheet/chartsheet.pyi', 'chartsheet/custom.pyi', 'chartsheet/properties.pyi', 'chartsheet/protection.pyi', 'chartsheet/publish.pyi', 'chartsheet/relation.pyi', 'chartsheet/views.pyi', 'comments/__init__.pyi', 'comments/author.pyi', 'comments/comment_sheet.pyi', 'comments/comments.pyi', 'comments/shape_writer.pyi', 'compat/__init__.pyi', 'compat/abc.pyi', 'compat/numbers.pyi', 'compat/product.pyi', 'compat/singleton.pyi', 'compat/strings.pyi', 'descriptors/__init__.pyi', 'descriptors/base.pyi', 'descriptors/excel.pyi', 'descriptors/namespace.pyi', 'descriptors/nested.pyi', 'descriptors/sequence.pyi', 'descriptors/serialisable.pyi', 'descriptors/slots.pyi', 'drawing/__init__.pyi', 'drawing/colors.pyi', 'drawing/connector.pyi', 'drawing/drawing.pyi', 'drawing/effect.pyi', 'drawing/fill.pyi', 'drawing/geometry.pyi', 'drawing/graphic.pyi', 'drawing/image.pyi', 'drawing/line.pyi', 'drawing/picture.pyi', 'drawing/properties.pyi', 'drawing/relation.pyi', 'drawing/spreadsheet_drawing.pyi', 'drawing/text.pyi', 'drawing/xdr.pyi', 'formatting/__init__.pyi', 'formatting/formatting.pyi', 'formatting/rule.pyi', 'formula/__init__.pyi', 'formula/tokenizer.pyi', 'formula/translate.pyi', 'packaging/__init__.pyi', 'packaging/core.pyi', 'packaging/custom.pyi', 'packaging/extended.pyi', 'packaging/interface.pyi', 'packaging/manifest.pyi', 'packaging/relationship.pyi', 'packaging/workbook.pyi', 'pivot/__init__.pyi', 'pivot/cache.pyi', 'pivot/fields.pyi', 'pivot/record.pyi', 'pivot/table.pyi', 'reader/__init__.pyi', 'reader/drawings.pyi', 'reader/excel.pyi', 'reader/strings.pyi', 'reader/workbook.pyi', 'styles/__init__.pyi', 'styles/alignment.pyi', 'styles/borders.pyi', 'styles/builtins.pyi', 'styles/cell_style.pyi', 'styles/colors.pyi', 'styles/differential.pyi', 'styles/fills.pyi', 'styles/fonts.pyi', 'styles/named_styles.pyi', 'styles/numbers.pyi', 'styles/protection.pyi', 'styles/proxy.pyi', 'styles/styleable.pyi', 'styles/stylesheet.pyi', 'styles/table.pyi', 'utils/__init__.pyi', 'utils/bound_dictionary.pyi', 'utils/cell.pyi', 'utils/dataframe.pyi', 'utils/datetime.pyi', 'utils/escape.pyi', 'utils/exceptions.pyi', 'utils/formulas.pyi', 'utils/indexed_list.pyi', 'utils/inference.pyi', 'utils/protection.pyi', 'utils/units.pyi', 'workbook/__init__.pyi', 'workbook/_writer.pyi', 'workbook/child.pyi', 'workbook/defined_name.pyi', 'workbook/external_link/__init__.pyi', 'workbook/external_link/external.pyi', 'workbook/external_reference.pyi', 'workbook/function_group.pyi', 'workbook/properties.pyi', 'workbook/protection.pyi', 'workbook/smart_tags.pyi', 'workbook/views.pyi', 'workbook/web.pyi', 'workbook/workbook.pyi', 'worksheet/__init__.pyi', 'worksheet/_read_only.pyi', 'worksheet/_reader.pyi', 'worksheet/_write_only.pyi', 'worksheet/_writer.pyi', 'worksheet/cell_range.pyi', 'worksheet/cell_watch.pyi', 'worksheet/controls.pyi', 'worksheet/copier.pyi', 'worksheet/custom.pyi', 'worksheet/datavalidation.pyi', 'worksheet/dimensions.pyi', 'worksheet/drawing.pyi', 'worksheet/errors.pyi', 'worksheet/filters.pyi', 'worksheet/formula.pyi', 'worksheet/header_footer.pyi', 'worksheet/hyperlink.pyi', 'worksheet/merge.pyi', 'worksheet/ole.pyi', 'worksheet/page.pyi', 'worksheet/pagebreak.pyi', 'worksheet/picture.pyi', 'worksheet/print_settings.pyi', 'worksheet/properties.pyi', 'worksheet/protection.pyi', 'worksheet/related.pyi', 'worksheet/scenario.pyi', 'worksheet/smart_tag.pyi', 'worksheet/table.pyi', 'worksheet/views.pyi', 'worksheet/worksheet.pyi', 'writer/__init__.pyi', 'writer/excel.pyi', 'writer/theme.pyi', 'xml/__init__.pyi', 'xml/_functions_overloads.pyi', 'xml/constants.pyi', 'xml/functions.pyi', 'METADATA.toml', 'py.typed']},
      license="Apache-2.0 license",
      python_requires=">=3.8",
      classifiers=[
          "License :: OSI Approved :: Apache Software License",
          "Programming Language :: Python :: 3",
          "Typing :: Stubs Only",
      ]
)
